@@ -1,8 +1,8 @@
 /*
  Copyright (C) Intel Corp.  2006.  All Rights Reserved.
- Intel funded Tungsten Graphics (http://www.tungstengraphics.com) to
+ Intel funded Tungsten Graphics to
  develop this 3D driver.
- 
+
  Permission is hereby granted, free of charge, to any person obtaining
  a copy of this software and associated documentation files (the
  "Software"), to deal in the Software without restriction, including
@@ -10,11 +10,11 @@
  distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to
  the following conditions:
- 
+
  The above copyright notice and this permission notice (including the
  next paragraph) shall be included in all copies or substantial
  portions of the Software.
- 
+
  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
@@ -22,13 +22,13 @@
  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
  OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
- 
+
  **********************************************************************/
  /*
   * Authors:
-  *   Keith Whitwell <keith@tungstengraphics.com>
+  *   Keith Whitwell <keithw@vmware.com>
   */
-   
+
 
 #include "main/glheader.h"
 #include "main/macros.h"
@@ -43,6 +43,15 @@
 #include "brw_sf.h"
 
 
+/**
+ * Determine the vue slot corresponding to the given half of the given register.
+ */
+static inline int vert_reg_to_vue_slot(struct brw_sf_compile *c, GLuint reg,
+                                       int half)
+{
+   return (reg + c->urb_entry_read_offset) * 2 + half;
+}
+
 /**
  * Determine the varying corresponding to the given half of the given
  * register.  half=0 means the first half of a register, half=1 means the
@@ -51,10 +60,23 @@
 static inline int vert_reg_to_varying(struct brw_sf_compile *c, GLuint reg,
                                       int half)
 {
-   int vue_slot = (reg + c->urb_entry_read_offset) * 2 + half;
+   int vue_slot = vert_reg_to_vue_slot(c, reg, half);
    return c->vue_map.slot_to_varying[vue_slot];
 }
 
+/**
+ * Determine the register corresponding to the given vue slot
+ */
+static struct brw_reg get_vue_slot(struct brw_sf_compile *c,
+                                   struct brw_reg vert,
+                                   int vue_slot)
+{
+   GLuint off = vue_slot / 2 - c->urb_entry_read_offset;
+   GLuint sub = vue_slot % 2;
+
+   return brw_vec4_grf(vert.nr + off, sub * 4);
+}
+
 /**
  * Determine the register corresponding to the given varying.
  */
@@ -64,10 +86,7 @@ static struct brw_reg get_varying(struct brw_sf_compile *c,
 {
    int vue_slot = c->vue_map.varying_to_slot[varying];
    assert (vue_slot >= c->urb_entry_read_offset);
-   GLuint off = vue_slot / 2 - c->urb_entry_read_offset;
-   GLuint sub = vue_slot % 2;
-
-   return brw_vec4_grf(vert.nr + off, sub * 4);
+   return get_vue_slot(c, vert, vue_slot);
 }
 
 static bool
@@ -76,7 +95,7 @@ have_attr(struct brw_sf_compile *c, GLuint attr)
    return (c->key.attrs & BITFIELD64_BIT(attr)) ? 1 : 0;
 }
 
-/*********************************************************************** 
+/***********************************************************************
  * Twoside lighting
  */
 static void copy_bfc( struct brw_sf_compile *c,
@@ -88,7 +107,7 @@ static void copy_bfc( struct brw_sf_compile *c,
    for (i = 0; i < 2; i++) {
       if (have_attr(c, VARYING_SLOT_COL0+i) &&
 	  have_attr(c, VARYING_SLOT_BFC0+i))
-	 brw_MOV(p, 
+	 brw_MOV(p,
 		 get_varying(c, vert, VARYING_SLOT_COL0+i),
 		 get_varying(c, vert, VARYING_SLOT_BFC0+i));
    }
@@ -105,20 +124,19 @@ static void do_twoside_color( struct brw_sf_compile *c )
    if (c->key.primitive == SF_UNFILLED_TRIS)
       return;
 
-   /* XXX: What happens if BFC isn't present?  This could only happen
-    * for user-supplied vertex programs, as t_vp_build.c always does
-    * the right thing.
+   /* If the vertex shader provides backface color, do the selection. The VS
+    * promises to set up the front color if the backface color is provided, but
+    * it may contain junk if never written to.
     */
    if (!(have_attr(c, VARYING_SLOT_COL0) && have_attr(c, VARYING_SLOT_BFC0)) &&
        !(have_attr(c, VARYING_SLOT_COL1) && have_attr(c, VARYING_SLOT_BFC1)))
       return;
-   
+
    /* Need to use BRW_EXECUTE_4 and also do an 4-wide compare in order
     * to get all channels active inside the IF.  In the clipping code
     * we run with NoMask, so it's not an option and we can use
     * BRW_EXECUTE_1 for all comparisions.
     */
-   brw_push_insn_state(p);
    brw_CMP(p, vec4(brw_null_reg()), backface_conditional, c->det, brw_imm_f(0));
    brw_IF(p, BRW_EXECUTE_4);
    {
@@ -129,7 +147,6 @@ static void do_twoside_color( struct brw_sf_compile *c )
       }
    }
    brw_ENDIF(p);
-   brw_pop_insn_state(p);
 }
 
 
@@ -138,24 +155,34 @@ static void do_twoside_color( struct brw_sf_compile *c )
  * Flat shading
  */
 
-#define VARYING_SLOT_COLOR_BITS (BITFIELD64_BIT(VARYING_SLOT_COL0) | \
-                                 BITFIELD64_BIT(VARYING_SLOT_COL1))
-
-static void copy_colors( struct brw_sf_compile *c,
-		     struct brw_reg dst,
-		     struct brw_reg src)
+static void copy_flatshaded_attributes(struct brw_sf_compile *c,
+                                       struct brw_reg dst,
+                                       struct brw_reg src)
 {
    struct brw_compile *p = &c->func;
-   GLuint i;
+   int i;
 
-   for (i = VARYING_SLOT_COL0; i <= VARYING_SLOT_COL1; i++) {
-      if (have_attr(c,i))
-	 brw_MOV(p, 
-		 get_varying(c, dst, i),
-		 get_varying(c, src, i));
+   for (i = 0; i < c->vue_map.num_slots; i++) {
+      if (c->key.interpolation_mode.mode[i] == INTERP_QUALIFIER_FLAT) {
+         brw_MOV(p,
+                 get_vue_slot(c, dst, i),
+                 get_vue_slot(c, src, i));
+      }
    }
 }
 
+static int count_flatshaded_attributes(struct brw_sf_compile *c)
+{
+   int i;
+   int count = 0;
+
+   for (i = 0; i < c->vue_map.num_slots; i++)
+      if (c->key.interpolation_mode.mode[i] == INTERP_QUALIFIER_FLAT)
+         count++;
+
+   return count;
+}
+
 
 
 /* Need to use a computed jump to copy flatshaded attributes as the
@@ -166,13 +193,9 @@ static void do_flatshade_triangle( struct brw_sf_compile *c )
 {
    struct brw_compile *p = &c->func;
    struct brw_context *brw = p->brw;
-   struct brw_reg ip = brw_ip_reg();
-   GLuint nr = _mesa_bitcount_64(c->key.attrs & VARYING_SLOT_COLOR_BITS);
+   GLuint nr;
    GLuint jmpi = 1;
 
-   if (!nr)
-      return;
-
    /* Already done in clip program:
     */
    if (c->key.primitive == SF_UNFILLED_TRIS)
@@ -181,23 +204,21 @@ static void do_flatshade_triangle( struct brw_sf_compile *c )
    if (brw->gen == 5)
        jmpi = 2;
 
-   brw_push_insn_state(p);
-   
+   nr = count_flatshaded_attributes(c);
+
    brw_MUL(p, c->pv, c->pv, brw_imm_d(jmpi*(nr*2+1)));
-   brw_JMPI(p, ip, ip, c->pv);
+   brw_JMPI(p, c->pv, BRW_PREDICATE_NONE);
 
-   copy_colors(c, c->vert[1], c->vert[0]);
-   copy_colors(c, c->vert[2], c->vert[0]);
-   brw_JMPI(p, ip, ip, brw_imm_d(jmpi*(nr*4+1)));
+   copy_flatshaded_attributes(c, c->vert[1], c->vert[0]);
+   copy_flatshaded_attributes(c, c->vert[2], c->vert[0]);
+   brw_JMPI(p, brw_imm_d(jmpi*(nr*4+1)), BRW_PREDICATE_NONE);
 
-   copy_colors(c, c->vert[0], c->vert[1]);
-   copy_colors(c, c->vert[2], c->vert[1]);
-   brw_JMPI(p, ip, ip, brw_imm_d(jmpi*nr*2));
+   copy_flatshaded_attributes(c, c->vert[0], c->vert[1]);
+   copy_flatshaded_attributes(c, c->vert[2], c->vert[1]);
+   brw_JMPI(p, brw_imm_d(jmpi*nr*2), BRW_PREDICATE_NONE);
 
-   copy_colors(c, c->vert[0], c->vert[2]);
-   copy_colors(c, c->vert[1], c->vert[2]);
-
-   brw_pop_insn_state(p);
+   copy_flatshaded_attributes(c, c->vert[0], c->vert[2]);
+   copy_flatshaded_attributes(c, c->vert[1], c->vert[2]);
 }
 	
 
@@ -205,14 +226,10 @@ static void do_flatshade_line( struct brw_sf_compile *c )
 {
    struct brw_compile *p = &c->func;
    struct brw_context *brw = p->brw;
-   struct brw_reg ip = brw_ip_reg();
-   GLuint nr = _mesa_bitcount_64(c->key.attrs & VARYING_SLOT_COLOR_BITS);
+   GLuint nr;
    GLuint jmpi = 1;
 
-   if (!nr)
-      return;
-
-   /* Already done in clip program: 
+   /* Already done in clip program:
     */
    if (c->key.primitive == SF_UNFILLED_TRIS)
       return;
@@ -220,16 +237,14 @@ static void do_flatshade_line( struct brw_sf_compile *c )
    if (brw->gen == 5)
        jmpi = 2;
 
-   brw_push_insn_state(p);
-   
+   nr = count_flatshaded_attributes(c);
+
    brw_MUL(p, c->pv, c->pv, brw_imm_d(jmpi*(nr+1)));
-   brw_JMPI(p, ip, ip, c->pv);
-   copy_colors(c, c->vert[1], c->vert[0]);
+   brw_JMPI(p, c->pv, BRW_PREDICATE_NONE);
+   copy_flatshaded_attributes(c, c->vert[1], c->vert[0]);
 
-   brw_JMPI(p, ip, ip, brw_imm_ud(jmpi*nr));
-   copy_colors(c, c->vert[0], c->vert[1]);
-
-   brw_pop_insn_state(p);
+   brw_JMPI(p, brw_imm_ud(jmpi*nr), BRW_PREDICATE_NONE);
+   copy_flatshaded_attributes(c, c->vert[0], c->vert[1]);
 }
 
 	
@@ -260,7 +275,7 @@ static void alloc_regs( struct brw_sf_compile *c )
    c->inv_w[1] = brw_vec1_grf(2, 3);
    c->z[2]     = brw_vec1_grf(2, 4);
    c->inv_w[2] = brw_vec1_grf(2, 5);
-   
+
    /* The vertices:
     */
    reg = 3;
@@ -279,7 +294,7 @@ static void alloc_regs( struct brw_sf_compile *c )
    /* Note grf allocation:
     */
    c->prog_data.total_grf = reg;
-   
+
 
    /* Outputs of this program - interpolation coefficients for
     * rasterization:
@@ -295,14 +310,10 @@ static void copy_z_inv_w( struct brw_sf_compile *c )
    struct brw_compile *p = &c->func;
    GLuint i;
 
-   brw_push_insn_state(p);
-	
    /* Copy both scalars with a single MOV:
     */
    for (i = 0; i < c->nr_verts; i++)
       brw_MOV(p, vec2(suboffset(c->vert[i], 2)), vec2(c->z[i]));
-	 
-   brw_pop_insn_state(p);
 }
 
 
@@ -311,49 +322,36 @@ static void invert_det( struct brw_sf_compile *c)
    /* Looks like we invert all 8 elements just to get 1/det in
     * position 2 !?!
     */
-   brw_math(&c->func, 
-	    c->inv_det, 
-	    BRW_MATH_FUNCTION_INV,
-	    0, 
-	    c->det,
-	    BRW_MATH_DATA_SCALAR,
-	    BRW_MATH_PRECISION_FULL);
+   gen4_math(&c->func,
+	     c->inv_det,
+	     BRW_MATH_FUNCTION_INV,
+	     0,
+	     c->det,
+	     BRW_MATH_DATA_SCALAR,
+	     BRW_MATH_PRECISION_FULL);
 
 }
 
 
 static bool
 calculate_masks(struct brw_sf_compile *c,
-	        GLuint reg,
-		GLushort *pc,
-		GLushort *pc_persp,
-		GLushort *pc_linear)
+                GLuint reg,
+                GLushort *pc,
+                GLushort *pc_persp,
+                GLushort *pc_linear)
 {
    bool is_last_attr = (reg == c->nr_setup_regs - 1);
-   GLbitfield64 persp_mask;
-   GLbitfield64 linear_mask;
-
-   if (c->key.do_flat_shading)
-      persp_mask = c->key.attrs & ~(BITFIELD64_BIT(VARYING_SLOT_POS) |
-                                    BITFIELD64_BIT(VARYING_SLOT_COL0) |
-                                    BITFIELD64_BIT(VARYING_SLOT_COL1));
-   else
-      persp_mask = c->key.attrs & ~(BITFIELD64_BIT(VARYING_SLOT_POS));
-
-   if (c->key.do_flat_shading)
-      linear_mask = c->key.attrs & ~(BITFIELD64_BIT(VARYING_SLOT_COL0) |
-                                     BITFIELD64_BIT(VARYING_SLOT_COL1));
-   else
-      linear_mask = c->key.attrs;
+   enum glsl_interp_qualifier interp;
 
    *pc_persp = 0;
    *pc_linear = 0;
    *pc = 0xf;
-      
-   if (persp_mask & BITFIELD64_BIT(vert_reg_to_varying(c, reg, 0)))
-      *pc_persp = 0xf;
 
-   if (linear_mask & BITFIELD64_BIT(vert_reg_to_varying(c, reg, 0)))
+   interp = c->key.interpolation_mode.mode[vert_reg_to_vue_slot(c, reg, 0)];
+   if (interp == INTERP_QUALIFIER_SMOOTH) {
+      *pc_linear = 0xf;
+      *pc_persp = 0xf;
+   } else if (interp == INTERP_QUALIFIER_NOPERSPECTIVE)
       *pc_linear = 0xf;
 
    /* Maybe only processs one attribute on the final round:
@@ -361,11 +359,12 @@ calculate_masks(struct brw_sf_compile *c,
    if (vert_reg_to_varying(c, reg, 1) != BRW_VARYING_SLOT_COUNT) {
       *pc |= 0xf0;
 
-      if (persp_mask & BITFIELD64_BIT(vert_reg_to_varying(c, reg, 1)))
-	 *pc_persp |= 0xf0;
-
-      if (linear_mask & BITFIELD64_BIT(vert_reg_to_varying(c, reg, 1)))
-	 *pc_linear |= 0xf0;
+      interp = c->key.interpolation_mode.mode[vert_reg_to_vue_slot(c, reg, 1)];
+      if (interp == INTERP_QUALIFIER_SMOOTH) {
+         *pc_linear |= 0xf0;
+         *pc_persp |= 0xf0;
+      } else if (interp == INTERP_QUALIFIER_NOPERSPECTIVE)
+         *pc_linear |= 0xf0;
    }
 
    return is_last_attr;
@@ -400,13 +399,29 @@ calculate_point_sprite_mask(struct brw_sf_compile *c, GLuint reg)
    return pc;
 }
 
+static void
+set_predicate_control_flag_value(struct brw_compile *p,
+                                 struct brw_sf_compile *c,
+                                 unsigned value)
+{
+   brw_set_default_predicate_control(p, BRW_PREDICATE_NONE);
 
+   if (value != 0xff) {
+      if (value != c->flag_value) {
+         brw_MOV(p, brw_flag_reg(0, 0), brw_imm_uw(value));
+         c->flag_value = value;
+      }
+
+      brw_set_default_predicate_control(p, BRW_PREDICATE_NORMAL);
+   }
+}
 
 void brw_emit_tri_setup(struct brw_sf_compile *c, bool allocate)
 {
    struct brw_compile *p = &c->func;
    GLuint i;
 
+   c->flag_value = 0xff;
    c->nr_verts = 3;
 
    if (allocate)
@@ -415,13 +430,13 @@ void brw_emit_tri_setup(struct brw_sf_compile *c, bool allocate)
    invert_det(c);
    copy_z_inv_w(c);
 
-   if (c->key.do_twoside_color) 
+   if (c->key.do_twoside_color)
       do_twoside_color(c);
 
-   if (c->key.do_flat_shading)
+   if (c->has_flat_shading)
       do_flatshade_triangle(c);
-      
-   
+
+
    for (i = 0; i < c->nr_setup_regs; i++)
    {
       /* Pair of incoming attributes:
@@ -434,18 +449,18 @@ void brw_emit_tri_setup(struct brw_sf_compile *c, bool allocate)
 
       if (pc_persp)
       {
-	 brw_set_predicate_control_flag_value(p, pc_persp);
+	 set_predicate_control_flag_value(p, c, pc_persp);
 	 brw_MUL(p, a0, a0, c->inv_w[0]);
 	 brw_MUL(p, a1, a1, c->inv_w[1]);
 	 brw_MUL(p, a2, a2, c->inv_w[2]);
       }
-      
-      
+
+
       /* Calculate coefficients for interpolated values:
-       */      
+       */
       if (pc_linear)
       {
-	 brw_set_predicate_control_flag_value(p, pc_linear);
+	 set_predicate_control_flag_value(p, c, pc_linear);
 
 	 brw_ADD(p, c->a1_sub_a0, a1, negate(a0));
 	 brw_ADD(p, c->a2_sub_a0, a2, negate(a0));
@@ -464,28 +479,28 @@ void brw_emit_tri_setup(struct brw_sf_compile *c, bool allocate)
       }
 
       {
-	 brw_set_predicate_control_flag_value(p, pc); 
+	 set_predicate_control_flag_value(p, c, pc);
 	 /* start point for interpolation
 	  */
 	 brw_MOV(p, c->m3C0, a0);
-      
+
 	 /* Copy m0..m3 to URB.  m0 is implicitly copied from r0 in
 	  * the send instruction:
-	  */	 
-	 brw_urb_WRITE(p, 
+	  */	
+	 brw_urb_WRITE(p,
 		       brw_null_reg(),
 		       0,
 		       brw_vec8_grf(0, 0), /* r0, will be copied to m0 */
-		       0, 	/* allocate */
-		       1,	/* used */
+                       last ? BRW_URB_WRITE_EOT_COMPLETE
+                       : BRW_URB_WRITE_NO_FLAGS,
 		       4, 	/* msg len */
 		       0,	/* response len */
-		       last,	/* eot */
-		       last, 	/* writes complete */
 		       i*4,	/* offset */
 		       BRW_URB_SWIZZLE_TRANSPOSE); /* XXX: Swizzle control "SF to windower" */
       }
    }
+
+   brw_set_default_predicate_control(p, BRW_PREDICATE_NONE);
 }
 
 
@@ -495,7 +510,7 @@ void brw_emit_line_setup(struct brw_sf_compile *c, bool allocate)
    struct brw_compile *p = &c->func;
    GLuint i;
 
-
+   c->flag_value = 0xff;
    c->nr_verts = 2;
 
    if (allocate)
@@ -504,7 +519,7 @@ void brw_emit_line_setup(struct brw_sf_compile *c, bool allocate)
    invert_det(c);
    copy_z_inv_w(c);
 
-   if (c->key.do_flat_shading)
+   if (c->has_flat_shading)
       do_flatshade_line(c);
 
    for (i = 0; i < c->nr_setup_regs; i++)
@@ -518,7 +533,7 @@ void brw_emit_line_setup(struct brw_sf_compile *c, bool allocate)
 
       if (pc_persp)
       {
-	 brw_set_predicate_control_flag_value(p, pc_persp);
+	 set_predicate_control_flag_value(p, c, pc_persp);
 	 brw_MUL(p, a0, a0, c->inv_w[0]);
 	 brw_MUL(p, a1, a1, c->inv_w[1]);
       }
@@ -526,11 +541,11 @@ void brw_emit_line_setup(struct brw_sf_compile *c, bool allocate)
       /* Calculate coefficients for position, color:
        */
       if (pc_linear) {
-	 brw_set_predicate_control_flag_value(p, pc_linear); 
+	 set_predicate_control_flag_value(p, c, pc_linear);
 
 	 brw_ADD(p, c->a1_sub_a0, a1, negate(a0));
 
- 	 brw_MUL(p, c->tmp, c->a1_sub_a0, c->dx0); 
+ 	 brw_MUL(p, c->tmp, c->a1_sub_a0, c->dx0);
 	 brw_MUL(p, c->m1Cx, c->tmp, c->inv_det);
 		
 	 brw_MUL(p, c->tmp, c->a1_sub_a0, c->dy0);
@@ -538,28 +553,28 @@ void brw_emit_line_setup(struct brw_sf_compile *c, bool allocate)
       }
 
       {
-	 brw_set_predicate_control_flag_value(p, pc); 
+	 set_predicate_control_flag_value(p, c, pc);
 
 	 /* start point for interpolation
 	  */
 	 brw_MOV(p, c->m3C0, a0);
 
-	 /* Copy m0..m3 to URB. 
+	 /* Copy m0..m3 to URB.
 	  */
-	 brw_urb_WRITE(p, 
+	 brw_urb_WRITE(p,
 		       brw_null_reg(),
 		       0,
 		       brw_vec8_grf(0, 0),
-		       0, 	/* allocate */
-		       1, 	/* used */
+                       last ? BRW_URB_WRITE_EOT_COMPLETE
+                       : BRW_URB_WRITE_NO_FLAGS,
 		       4, 	/* msg len */
 		       0,	/* response len */
-		       last, 	/* eot */
-		       last, 	/* writes complete */
 		       i*4,	/* urb destination offset */
-		       BRW_URB_SWIZZLE_TRANSPOSE); 
+		       BRW_URB_SWIZZLE_TRANSPOSE);
       }
-   } 
+   }
+
+   brw_set_default_predicate_control(p, BRW_PREDICATE_NONE);
 }
 
 void brw_emit_point_sprite_setup(struct brw_sf_compile *c, bool allocate)
@@ -567,6 +582,7 @@ void brw_emit_point_sprite_setup(struct brw_sf_compile *c, bool allocate)
    struct brw_compile *p = &c->func;
    GLuint i;
 
+   c->flag_value = 0xff;
    c->nr_verts = 1;
 
    if (allocate)
@@ -583,7 +599,7 @@ void brw_emit_point_sprite_setup(struct brw_sf_compile *c, bool allocate)
       pc_persp &= ~pc_coord_replace;
 
       if (pc_persp) {
-	 brw_set_predicate_control_flag_value(p, pc_persp);
+	 set_predicate_control_flag_value(p, c, pc_persp);
 	 brw_MUL(p, a0, a0, c->inv_w[0]);
       }
 
@@ -593,17 +609,17 @@ void brw_emit_point_sprite_setup(struct brw_sf_compile *c, bool allocate)
        * point.
        */
       if (pc_coord_replace) {
-	 brw_set_predicate_control_flag_value(p, pc_coord_replace);
+	 set_predicate_control_flag_value(p, c, pc_coord_replace);
 	 /* Caculate 1.0/PointWidth */
-	 brw_math(&c->func,
-		  c->tmp,
-		  BRW_MATH_FUNCTION_INV,
-		  0,
-		  c->dx0,
-		  BRW_MATH_DATA_SCALAR,
-		  BRW_MATH_PRECISION_FULL);
+	 gen4_math(&c->func,
+		   c->tmp,
+		   BRW_MATH_FUNCTION_INV,
+		   0,
+		   c->dx0,
+		   BRW_MATH_DATA_SCALAR,
+		   BRW_MATH_PRECISION_FULL);
 
-	 brw_set_access_mode(p, BRW_ALIGN_16);
+	 brw_set_default_access_mode(p, BRW_ALIGN_16);
 
 	 /* dA/dx, dA/dy */
 	 brw_MOV(p, c->m1Cx, brw_imm_f(0.0));
@@ -623,32 +639,32 @@ void brw_emit_point_sprite_setup(struct brw_sf_compile *c, bool allocate)
 	    brw_MOV(p, brw_writemask(c->m3C0, WRITEMASK_W), brw_imm_f(1.0));
 	 }
 
-	 brw_set_access_mode(p, BRW_ALIGN_1);
+	 brw_set_default_access_mode(p, BRW_ALIGN_1);
       }
 
       if (pc & ~pc_coord_replace) {
-	 brw_set_predicate_control_flag_value(p, pc & ~pc_coord_replace);
+	 set_predicate_control_flag_value(p, c, pc & ~pc_coord_replace);
 	 brw_MOV(p, c->m1Cx, brw_imm_ud(0));
 	 brw_MOV(p, c->m2Cy, brw_imm_ud(0));
 	 brw_MOV(p, c->m3C0, a0); /* constant value */
       }
 
 
-      brw_set_predicate_control_flag_value(p, pc);
+      set_predicate_control_flag_value(p, c, pc);
       /* Copy m0..m3 to URB. */
       brw_urb_WRITE(p,
 		    brw_null_reg(),
 		    0,
 		    brw_vec8_grf(0, 0),
-		    0, 	/* allocate */
-		    1,	/* used */
+                    last ? BRW_URB_WRITE_EOT_COMPLETE
+                    : BRW_URB_WRITE_NO_FLAGS,
 		    4, 	/* msg len */
 		    0,	/* response len */
-		    last, 	/* eot */
-		    last, 	/* writes complete */
 		    i*4,	/* urb destination offset */
 		    BRW_URB_SWIZZLE_TRANSPOSE);
    }
+
+   brw_set_default_predicate_control(p, BRW_PREDICATE_NONE);
 }
 
 /* Points setup - several simplifications as all attributes are
@@ -659,8 +675,9 @@ void brw_emit_point_setup(struct brw_sf_compile *c, bool allocate)
    struct brw_compile *p = &c->func;
    GLuint i;
 
+   c->flag_value = 0xff;
    c->nr_verts = 1;
-   
+
    if (allocate)
       alloc_regs(c);
 
@@ -674,13 +691,13 @@ void brw_emit_point_setup(struct brw_sf_compile *c, bool allocate)
       struct brw_reg a0 = offset(c->vert[0], i);
       GLushort pc, pc_persp, pc_linear;
       bool last = calculate_masks(c, i, &pc, &pc_persp, &pc_linear);
-            
+
       if (pc_persp)
       {				
 	 /* This seems odd as the values are all constant, but the
 	  * fragment shader will be expecting it:
 	  */
-	 brw_set_predicate_control_flag_value(p, pc_persp);
+	 set_predicate_control_flag_value(p, c, pc_persp);
 	 brw_MUL(p, a0, a0, c->inv_w[0]);
       }
 
@@ -690,39 +707,37 @@ void brw_emit_point_setup(struct brw_sf_compile *c, bool allocate)
        * code in the fragment shader.
        */
       {
-	 brw_set_predicate_control_flag_value(p, pc); 
+	 set_predicate_control_flag_value(p, c, pc);
 
 	 brw_MOV(p, c->m3C0, a0); /* constant value */
 
-	 /* Copy m0..m3 to URB. 
+	 /* Copy m0..m3 to URB.
 	  */
-	 brw_urb_WRITE(p, 
+	 brw_urb_WRITE(p,
 		       brw_null_reg(),
 		       0,
 		       brw_vec8_grf(0, 0),
-		       0, 	/* allocate */
-		       1,	/* used */
+                       last ? BRW_URB_WRITE_EOT_COMPLETE
+                       : BRW_URB_WRITE_NO_FLAGS,
 		       4, 	/* msg len */
 		       0,	/* response len */
-		       last, 	/* eot */
-		       last, 	/* writes complete */
 		       i*4,	/* urb destination offset */
 		       BRW_URB_SWIZZLE_TRANSPOSE);
       }
    }
+
+   brw_set_default_predicate_control(p, BRW_PREDICATE_NONE);
 }
 
 void brw_emit_anyprim_setup( struct brw_sf_compile *c )
 {
    struct brw_compile *p = &c->func;
-   struct brw_reg ip = brw_ip_reg();
+   struct brw_context *brw = p->brw;
    struct brw_reg payload_prim = brw_uw1_reg(BRW_GENERAL_REGISTER_FILE, 1, 0);
-   struct brw_reg payload_attr = get_element_ud(brw_vec1_reg(BRW_GENERAL_REGISTER_FILE, 1, 0), 0); 
+   struct brw_reg payload_attr = get_element_ud(brw_vec1_reg(BRW_GENERAL_REGISTER_FILE, 1, 0), 0);
    struct brw_reg primmask;
    int jmp;
    struct brw_reg v1_null_ud = vec1(retype(brw_null_reg(), BRW_REGISTER_TYPE_UD));
-   
-   GLuint saveflag;
 
    c->nr_verts = 3;
    alloc_regs(c);
@@ -732,7 +747,6 @@ void brw_emit_anyprim_setup( struct brw_sf_compile *c )
    brw_MOV(p, primmask, brw_imm_ud(1));
    brw_SHL(p, primmask, primmask, payload_prim);
 
-   brw_set_conditionalmod(p, BRW_CONDITIONAL_Z);
    brw_AND(p, v1_null_ud, primmask, brw_imm_ud((1<<_3DPRIM_TRILIST) |
 					       (1<<_3DPRIM_TRISTRIP) |
 					       (1<<_3DPRIM_TRIFAN) |
@@ -740,49 +754,27 @@ void brw_emit_anyprim_setup( struct brw_sf_compile *c )
 					       (1<<_3DPRIM_POLYGON) |
 					       (1<<_3DPRIM_RECTLIST) |
 					       (1<<_3DPRIM_TRIFAN_NOSTIPPLE)));
-   jmp = brw_JMPI(p, ip, ip, brw_imm_d(0)) - p->store;
-   {
-      saveflag = p->flag_value;
-      brw_push_insn_state(p); 
-      brw_emit_tri_setup( c, false );
-      brw_pop_insn_state(p);
-      p->flag_value = saveflag;
-      /* note - thread killed in subroutine, so must
-       * restore the flag which is changed when building
-       * the subroutine. fix #13240
-       */
-   }
+   brw_inst_set_cond_modifier(brw, brw_last_inst, BRW_CONDITIONAL_Z);
+   jmp = brw_JMPI(p, brw_imm_d(0), BRW_PREDICATE_NORMAL) - p->store;
+   brw_emit_tri_setup(c, false);
    brw_land_fwd_jump(p, jmp);
 
-   brw_set_conditionalmod(p, BRW_CONDITIONAL_Z);
    brw_AND(p, v1_null_ud, primmask, brw_imm_ud((1<<_3DPRIM_LINELIST) |
 					       (1<<_3DPRIM_LINESTRIP) |
 					       (1<<_3DPRIM_LINELOOP) |
 					       (1<<_3DPRIM_LINESTRIP_CONT) |
 					       (1<<_3DPRIM_LINESTRIP_BF) |
 					       (1<<_3DPRIM_LINESTRIP_CONT_BF)));
-   jmp = brw_JMPI(p, ip, ip, brw_imm_d(0)) - p->store;
-   {
-      saveflag = p->flag_value;
-      brw_push_insn_state(p); 
-      brw_emit_line_setup( c, false );
-      brw_pop_insn_state(p);
-      p->flag_value = saveflag;
-      /* note - thread killed in subroutine */
-   }
-   brw_land_fwd_jump(p, jmp); 
+   brw_inst_set_cond_modifier(brw, brw_last_inst, BRW_CONDITIONAL_Z);
+   jmp = brw_JMPI(p, brw_imm_d(0), BRW_PREDICATE_NORMAL) - p->store;
+   brw_emit_line_setup(c, false);
+   brw_land_fwd_jump(p, jmp);
 
-   brw_set_conditionalmod(p, BRW_CONDITIONAL_Z);
    brw_AND(p, v1_null_ud, payload_attr, brw_imm_ud(1<<BRW_SPRITE_POINT_ENABLE));
-   jmp = brw_JMPI(p, ip, ip, brw_imm_d(0)) - p->store;
-   {
-      saveflag = p->flag_value;
-      brw_push_insn_state(p); 
-      brw_emit_point_sprite_setup( c, false );
-      brw_pop_insn_state(p);
-      p->flag_value = saveflag;
-   }
-   brw_land_fwd_jump(p, jmp); 
+   brw_inst_set_cond_modifier(brw, brw_last_inst, BRW_CONDITIONAL_Z);
+   jmp = brw_JMPI(p, brw_imm_d(0), BRW_PREDICATE_NORMAL) - p->store;
+   brw_emit_point_sprite_setup(c, false);
+   brw_land_fwd_jump(p, jmp);
 
    brw_emit_point_setup( c, false );
 }
